@@ -5,35 +5,47 @@ Implementation of validate_schema(schema)
 
 
 import datetime
+import math
+import string
 import typing
 import urllib.parse
 import re
 
+import pint
+
 from ..errors import ValidationError, ActionDoesNotExistError, InvalidNumberError, InvalidTemplateIDError, RecursiveTemplateError
 from .utils import units_are_valid
 from .validate import validate
 from .templates import substitute_templates
 from .conditions import validate_condition_schema
-from ..languages import get_languages
+from ..languages import get_language_codes
+from .. import datatypes
 
 __author__ = 'Florian Rhiem <f.rhiem@fz-juelich.de>'
 
 
 def validate_schema(
-        schema: dict,
+        schema: typing.Dict[str, typing.Any],
         path: typing.Optional[typing.List[str]] = None,
         *,
-        parent_conditions: typing.Optional[typing.List[typing.Tuple]] = None,
-        invalid_template_action_ids: typing.Sequence[int] = ()
+        parent_conditions: typing.Optional[typing.List[typing.Tuple[typing.List[str], typing.Dict[str, typing.Any]]]] = None,
+        invalid_template_action_ids: typing.Sequence[int] = (),
+        strict: bool = False,
+        all_language_codes: typing.Optional[typing.Set[str]] = None
 ) -> None:
     """
     Validates the given schema and raises a ValidationError if it is invalid.
 
     :param schema: the sampledb object schema
     :param path: the path to this subschema
+    :param parent_conditions: conditions defined in parent objects
     :param invalid_template_action_ids: IDs of actions that may not be used as templates to prevent recursion
+    :param strict: whether the schema should be evaluated in strict mode, or backwards compatible otherwise
+    :param all_language_codes: the set of existing language codes, or None
     :raise ValidationError: if the schema is invalid.
     """
+    if all_language_codes is None:
+        all_language_codes = get_language_codes()
     if path is None:
         path = []
     if not isinstance(schema, dict):
@@ -44,10 +56,6 @@ def validate_schema(
         raise ValidationError('invalid schema (type must be str)', path)
     if 'title' not in schema:
         raise ValidationError('invalid schema (must contain title)', path)
-    all_language_codes = {
-        language.lang_code
-        for language in get_languages()
-    }
     if not isinstance(schema['title'], str) and not isinstance(schema['title'], dict):
         raise ValidationError('title must be str or dict', path)
     if isinstance(schema['title'], dict):
@@ -75,40 +83,41 @@ def validate_schema(
     if path == [] and schema['type'] != 'object':
         raise ValidationError('invalid schema (root must be an object)', path)
     if schema['type'] == 'array':
-        return _validate_array_schema(schema, path, invalid_template_action_ids)
+        return _validate_array_schema(schema, path, invalid_template_action_ids=invalid_template_action_ids, strict=strict, all_language_codes=all_language_codes)
     elif schema['type'] == 'object':
-        return _validate_object_schema(schema, path, invalid_template_action_ids)
+        return _validate_object_schema(schema, path, invalid_template_action_ids=invalid_template_action_ids, strict=strict, all_language_codes=all_language_codes)
     elif schema['type'] == 'text':
-        return _validate_text_schema(schema, path)
+        return _validate_text_schema(schema, path, all_language_codes=all_language_codes)
     elif schema['type'] == 'datetime':
-        return _validate_datetime_schema(schema, path)
+        return _validate_datetime_schema(schema, path, all_language_codes=all_language_codes)
     elif schema['type'] == 'bool':
-        return _validate_bool_schema(schema, path)
+        return _validate_bool_schema(schema, path, all_language_codes=all_language_codes)
     elif schema['type'] == 'quantity':
-        return _validate_quantity_schema(schema, path)
+        return _validate_quantity_schema(schema, path, all_language_codes=all_language_codes)
     elif schema['type'] == 'sample':
-        return _validate_sample_schema(schema, path)
+        return _validate_sample_schema(schema, path, all_language_codes=all_language_codes)
     elif schema['type'] == 'measurement':
-        return _validate_measurement_schema(schema, path)
+        return _validate_measurement_schema(schema, path, all_language_codes=all_language_codes)
     elif schema['type'] == 'object_reference':
-        return _validate_object_reference_schema(schema, path)
+        return _validate_object_reference_schema(schema, path, all_language_codes=all_language_codes)
     elif schema['type'] == 'tags':
-        return _validate_tags_schema(schema, path)
+        return _validate_tags_schema(schema, path, strict=strict)
     elif schema['type'] == 'hazards':
-        return _validate_hazards_schema(schema, path)
+        return _validate_hazards_schema(schema, path, all_language_codes=all_language_codes)
     elif schema['type'] == 'user':
-        return _validate_user_schema(schema, path)
+        return _validate_user_schema(schema, path, all_language_codes=all_language_codes)
     elif schema['type'] == 'plotly_chart':
-        return _validate_plotly_chart_schema(schema, path)
+        return _validate_plotly_chart_schema(schema, path, all_language_codes=all_language_codes)
     else:
         raise ValidationError('invalid type', path)
 
 
-def _validate_note_in_schema(schema: dict, path: typing.List[str]):
-    all_language_codes = {
-        language.lang_code
-        for language in get_languages()
-    }
+def _validate_note_in_schema(
+        schema: typing.Dict[str, typing.Any],
+        path: typing.List[str],
+        *,
+        all_language_codes: typing.Set[str]
+) -> None:
     if 'note' in schema and not isinstance(schema['note'], str) and not isinstance(schema['note'], dict):
         raise ValidationError('note must be str or dict', path)
     if 'note' in schema and isinstance(schema['note'], dict):
@@ -122,15 +131,21 @@ def _validate_note_in_schema(schema: dict, path: typing.List[str]):
                 raise ValidationError('note must only contain text', path)
 
 
-def _validate_hazards_schema(schema: dict, path: typing.List[str]) -> None:
+def _validate_hazards_schema(
+        schema: typing.Dict[str, typing.Any],
+        path: typing.List[str],
+        *,
+        all_language_codes: typing.Set[str]
+) -> None:
     """
     Validate the given GHS hazards schema and raise a ValidationError if it is invalid.
 
     :param schema: the sampledb object schema
     :param path: the path to this subschema
+    :param all_language_codes: the set of existing language codes
     :raise ValidationError: if the schema is invalid.
     """
-    valid_keys = {'type', 'title', 'note', 'dataverse_export', 'may_copy', 'style'}
+    valid_keys = {'type', 'title', 'note', 'dataverse_export', 'scicat_export', 'may_copy', 'style'}
     required_keys = {'type', 'title'}
     schema_keys = set(schema.keys())
     invalid_keys = schema_keys - valid_keys
@@ -141,16 +156,21 @@ def _validate_hazards_schema(schema: dict, path: typing.List[str]) -> None:
         raise ValidationError('missing keys in schema: {}'.format(missing_keys), path)
     if 'dataverse_export' in schema and not isinstance(schema['dataverse_export'], bool):
         raise ValidationError('dataverse_export must be True or False', path)
+    if 'scicat_export' in schema and not isinstance(schema['scicat_export'], bool):
+        raise ValidationError('scicat_export must be True or False', path)
     if path != ['hazards']:
         raise ValidationError('GHS hazards must be a top-level entry named "hazards"', path)
 
-    _validate_note_in_schema(schema, path)
+    _validate_note_in_schema(schema, path, all_language_codes=all_language_codes)
 
 
 def _validate_array_schema(
-        schema: dict,
+        schema: typing.Dict[str, typing.Any],
         path: typing.List[str],
-        invalid_template_action_ids: typing.Sequence[int] = ()
+        *,
+        invalid_template_action_ids: typing.Sequence[int] = (),
+        strict: bool = False,
+        all_language_codes: typing.Set[str]
 ) -> None:
     """
     Validates the given array schema and raises a ValidationError if it is invalid.
@@ -158,9 +178,11 @@ def _validate_array_schema(
     :param schema: the sampledb object schema
     :param path: the path to this subschema
     :param invalid_template_action_ids: IDs of actions that may not be used as templates to prevent recursion
+    :param strict: whether the schema should be evaluated in strict mode, or backwards compatible otherwise
+    :param all_language_codes: the set of existing language codes
     :raise ValidationError: if the schema is invalid.
     """
-    valid_keys = {'type', 'title', 'items', 'style', 'minItems', 'maxItems', 'defaultItems', 'default', 'may_copy'}
+    valid_keys = {'type', 'title', 'items', 'style', 'minItems', 'maxItems', 'defaultItems', 'default', 'may_copy', 'conditions'}
     required_keys = {'type', 'title', 'items'}
     schema_keys = set(schema.keys())
     invalid_keys = schema_keys - valid_keys
@@ -202,22 +224,28 @@ def _validate_array_schema(
     if has_default_items and has_max_items:
         if schema['defaultItems'] > schema['maxItems']:
             raise ValidationError('defaultItems must be less than or equal to maxItems', path)
-    validate_schema(schema['items'], path + ['[?]'], invalid_template_action_ids=invalid_template_action_ids)
+    validate_schema(schema['items'], path + ['[?]'], invalid_template_action_ids=invalid_template_action_ids, strict=strict, all_language_codes=all_language_codes)
     if 'default' in schema:
         if has_default_items:
             raise ValidationError('default and defaultItems are mutually exclusive', path)
-        validate(schema['default'], schema, path + ['(default)'])
+        validate(schema['default'], schema, path + ['(default)'], strict=strict)
 
 
-def _validate_tags_schema(schema: dict, path: typing.List[str]) -> None:
+def _validate_tags_schema(
+        schema: typing.Dict[str, typing.Any],
+        path: typing.List[str],
+        *,
+        strict: bool = False
+) -> None:
     """
     Validates the given tags schema and raises a ValidationError if it is invalid.
 
     :param schema: the sampledb object schema
     :param path: the path to this subschema
+    :param strict: whether the schema should be evaluated in strict mode, or backwards compatible otherwise
     :raise ValidationError: if the schema is invalid.
     """
-    valid_keys = {'type', 'title', 'default', 'dataverse_export', 'may_copy', 'style'}
+    valid_keys = {'type', 'title', 'default', 'dataverse_export', 'scicat_export', 'may_copy', 'style'}
     required_keys = {'type', 'title'}
     schema_keys = set(schema.keys())
     invalid_keys = schema_keys - valid_keys
@@ -227,17 +255,22 @@ def _validate_tags_schema(schema: dict, path: typing.List[str]) -> None:
     if missing_keys:
         raise ValidationError('missing keys in schema: {}'.format(missing_keys), path)
     if 'default' in schema:
-        validate({'_type': 'tags', 'tags': schema['default']}, schema, path + ['(default)'])
+        validate({'_type': 'tags', 'tags': schema['default']}, schema, path + ['(default)'], strict=strict)
     if 'dataverse_export' in schema and not isinstance(schema['dataverse_export'], bool):
         raise ValidationError('dataverse_export must be True or False', path)
+    if 'scicat_export' in schema and not isinstance(schema['scicat_export'], bool):
+        raise ValidationError('scicat_export must be True or False', path)
     if path != ['tags']:
         raise ValidationError('Tags must be a top-level entry named "tags"', path)
 
 
 def _validate_object_schema(
-        schema: dict,
+        schema: typing.Dict[str, typing.Any],
         path: typing.List[str],
-        invalid_template_action_ids: typing.Sequence[int] = ()
+        *,
+        invalid_template_action_ids: typing.Sequence[int] = (),
+        strict: bool = False,
+        all_language_codes: typing.Set[str]
 ) -> None:
     """
     Validates the given object schema and raises a ValidationError if it is invalid.
@@ -245,6 +278,8 @@ def _validate_object_schema(
     :param schema: the sampledb object schema
     :param path: the path to this subschema
     :param invalid_template_action_ids: IDs of actions that may not be used as templates to prevent recursion
+    :param strict: whether the schema should be evaluated in strict mode, or backwards compatible otherwise
+    :param all_language_codes: the set of existing language codes
     :raise ValidationError: if the schema is invalid.
     """
     try:
@@ -259,15 +294,18 @@ def _validate_object_schema(
         raise ValidationError('template must be the ID of a template action', path)
 
     if schema.get('template') is not None:
-        invalid_template_action_ids = list(invalid_template_action_ids) + [schema.get('template')]
+        invalid_template_action_ids = list(invalid_template_action_ids) + [typing.cast(int, schema['template'])]
 
-    valid_keys = {'type', 'title', 'properties', 'propertyOrder', 'required', 'default', 'may_copy', 'style', 'template', 'note'}
+    valid_keys = {'type', 'title', 'properties', 'propertyOrder', 'required', 'default', 'may_copy', 'style', 'template', 'recipes', 'note', 'show_more'}
     if not path:
         # the top level object may contain a list of properties to be displayed in a table of objects
         valid_keys.add('displayProperties')
         valid_keys.add('batch')
         valid_keys.add('batch_name_format')
         valid_keys.add('notebookTemplates')
+    if path:
+        # the top level object must not have any conditions
+        valid_keys.add('conditions')
     required_keys = {'type', 'title', 'properties'}
     schema_keys = set(schema.keys())
     invalid_keys = schema_keys - valid_keys
@@ -279,16 +317,34 @@ def _validate_object_schema(
 
     if not isinstance(schema['properties'], dict):
         raise ValidationError('properties must be dict', path)
-    property_conditions = []
+    property_conditions: typing.List[typing.Tuple[typing.List[str], typing.Dict[str, typing.Any]]] = []
     property_schemas = {}
     for property_name, property_schema in schema['properties'].items():
+        property_name_valid = True
+        if not property_name:
+            property_name_valid = False
         if '__' in property_name:
+            property_name_valid = False
+        if strict and property_name_valid:
+            # property name may only consist of ascii characters, digits and underscores
+            if not all(c in (string.ascii_letters + string.digits + '_') for c in property_name):
+                property_name_valid = False
+            # property name must start with a character
+            if property_name[0] not in string.ascii_letters:
+                property_name_valid = False
+            # property name must not end with an underscore
+            if property_name.endswith('_'):
+                property_name_valid = False
+        if not property_name_valid:
             raise ValidationError('invalid property name: {}'.format(property_name), path)
+
         validate_schema(
             property_schema,
             path + [property_name],
             parent_conditions=property_conditions,
-            invalid_template_action_ids=invalid_template_action_ids
+            invalid_template_action_ids=invalid_template_action_ids,
+            strict=strict,
+            all_language_codes=all_language_codes
         )
         property_schemas[property_name] = property_schema
     for condition_path, condition in property_conditions:
@@ -329,7 +385,7 @@ def _validate_object_schema(
                 raise ValidationError('duplicate propertyOrder property: {}'.format(property_name), path)
 
     if 'default' in schema:
-        validate(schema['default'], schema)
+        validate(schema['default'], schema, strict=strict)
 
     if 'displayProperties' in schema:
         if not isinstance(schema['displayProperties'], list):
@@ -356,35 +412,72 @@ def _validate_object_schema(
 
     if 'notebookTemplates' in schema:
         _validate_notebook_templates(schema['notebookTemplates'])
-    _validate_note_in_schema(schema, path)
+
+    if 'recipes' in schema:
+        for recipe in schema['recipes']:
+            if 'name' not in recipe:
+                raise ValidationError('missing recipe name', path + ['(recipes)'])
+            if not isinstance(recipe['name'], str) and not isinstance(recipe['name'], dict):
+                raise ValidationError('recipe name must be str or dict', path + ['(recipes)'])
+            if isinstance(recipe['name'], dict):
+                if 'en' not in recipe['name']:
+                    raise ValidationError('recipe name must include an english translation', path + ['(recipes)'])
+                for lang_code in recipe['name'].keys():
+                    if lang_code not in all_language_codes:
+                        raise ValidationError('recipe name must only contain known languages', path + ['(recipes)'])
+                for name_text in recipe['name'].values():
+                    if not isinstance(name_text, str):
+                        raise ValidationError('recipe name must only contain text', path + ['(recipes)'])
+            if 'property_values' not in recipe:
+                raise ValidationError('missing property_values', path + ['(recipes)'])
+            for property_name in recipe['property_values']:
+                if property_name not in schema['properties'].keys():
+                    raise ValidationError('unknown property: {}'.format(property_name), path + ['(recipes)'])
+                if schema['properties'][property_name]['type'] not in ['text', 'quantity', 'datetime', 'bool']:
+                    raise ValidationError('unsupported type in recipe', path + ['(recipes)', property_name])
+                if recipe['property_values'][property_name] is not None:
+                    validate(recipe['property_values'][property_name], schema['properties'][property_name], path + ['(recipes)', property_name], strict=strict)
+                elif schema['properties'][property_name]['type'] == 'bool':
+                    raise ValidationError('recipe values for type \'bool\' must not be None', path + ['(recipes)', property_name])
+
+    if 'show_more' in schema:
+        if not isinstance(schema['show_more'], list):
+            raise ValidationError('show_more must be list', path)
+        for property_name in schema['show_more']:
+            if property_name not in schema['properties'].keys():
+                raise ValidationError('unknown property: {}'.format(property_name), path)
+
+    _validate_note_in_schema(schema, path, all_language_codes=all_language_codes)
 
 
-def _validate_text_schema(schema: dict, path: typing.List[str]) -> None:
+def _validate_text_schema(
+        schema: typing.Dict[str, typing.Any],
+        path: typing.List[str],
+        *,
+        all_language_codes: typing.Set[str]
+) -> None:
     """
     Validates the given text object schema and raises a ValidationError if it is invalid.
 
     :param schema: the sampledb object schema
     :param path: the path to this subschema
+    :param all_language_codes: the set of existing language codes
     :raise ValidationError: if the schema is invalid.
     """
-    valid_keys = {'type', 'title', 'default', 'minLength', 'maxLength', 'choices', 'pattern', 'multiline', 'markdown', 'note', 'placeholder', 'dataverse_export', 'languages', 'conditions', 'may_copy', 'style'}
+    valid_keys = {'type', 'title', 'default', 'minLength', 'maxLength', 'choices', 'pattern', 'multiline', 'markdown', 'note', 'placeholder', 'dataverse_export', 'scicat_export', 'languages', 'conditions', 'may_copy', 'style'}
     schema_keys = set(schema.keys())
     invalid_keys = schema_keys - valid_keys
     if invalid_keys:
         raise ValidationError('unexpected keys in schema: {}'.format(invalid_keys), path)
 
-    all_language_codes = {
-        language.lang_code
-        for language in get_languages()
-    }
     if 'languages' in schema:
         if schema['languages'] != 'all':
-            if not isinstance(schema['languages'], list):
-                raise ValidationError('languages must be a list of known language codes or "all"', path)
+            if not isinstance(schema['languages'], list) or len(schema['languages']) == 0:
+                raise ValidationError('languages must be a non-empty list of known language codes or "all"', path)
             for language in schema['languages']:
                 if not isinstance(language, str) or language not in all_language_codes:
                     raise ValidationError('languages must be a list of known language codes or "all"', path)
-            allowed_language_codes = schema['languages']
+            allowed_language_codes = set(schema['languages'])
         else:
             allowed_language_codes = all_language_codes
     else:
@@ -471,18 +564,28 @@ def _validate_text_schema(schema: dict, path: typing.List[str]) -> None:
         raise ValidationError('dataverse_export must be True or False', path)
     if 'dataverse_export' in schema and not schema['dataverse_export'] and path == ['name']:
         raise ValidationError('dataverse_export must be True for the object name', path)
-    _validate_note_in_schema(schema, path)
+    if 'scicat_export' in schema and not isinstance(schema['scicat_export'], bool):
+        raise ValidationError('scicat_export must be True or False', path)
+    if 'scicat_export' in schema and not schema['scicat_export'] and path == ['name']:
+        raise ValidationError('scicat_export must be True for the object name', path)
+    _validate_note_in_schema(schema, path, all_language_codes=all_language_codes)
 
 
-def _validate_datetime_schema(schema: dict, path: typing.List[str]) -> None:
+def _validate_datetime_schema(
+        schema: typing.Dict[str, typing.Any],
+        path: typing.List[str],
+        *,
+        all_language_codes: typing.Set[str]
+) -> None:
     """
     Validates the given datetime object schema and raises a ValidationError if it is invalid.
 
     :param schema: the sampledb object schema
     :param path: the path to this subschema
+    :param all_language_codes: the set of existing language codes
     :raise ValidationError: if the schema is invalid.
     """
-    valid_keys = {'type', 'title', 'default', 'note', 'dataverse_export', 'conditions', 'may_copy', 'style'}
+    valid_keys = {'type', 'title', 'default', 'note', 'dataverse_export', 'scicat_export', 'conditions', 'may_copy', 'style'}
     schema_keys = set(schema.keys())
     invalid_keys = schema_keys - valid_keys
     if invalid_keys:
@@ -498,10 +601,17 @@ def _validate_datetime_schema(schema: dict, path: typing.List[str]) -> None:
                 raise ValidationError('invalid default value', path)
     if 'dataverse_export' in schema and not isinstance(schema['dataverse_export'], bool):
         raise ValidationError('dataverse_export must be True or False', path)
-    _validate_note_in_schema(schema, path)
+    if 'scicat_export' in schema and not isinstance(schema['scicat_export'], bool):
+        raise ValidationError('scicat_export must be True or False', path)
+    _validate_note_in_schema(schema, path, all_language_codes=all_language_codes)
 
 
-def _validate_bool_schema(schema: dict, path: typing.List[str]) -> None:
+def _validate_bool_schema(
+        schema: typing.Dict[str, typing.Any],
+        path: typing.List[str],
+        *,
+        all_language_codes: typing.Set[str]
+) -> None:
     """
     Validates the given boolean object schema and raises a ValidationError if it is invalid.
 
@@ -509,7 +619,7 @@ def _validate_bool_schema(schema: dict, path: typing.List[str]) -> None:
     :param path: the path to this subschema
     :raise ValidationError: if the schema is invalid.
     """
-    valid_keys = {'type', 'title', 'default', 'note', 'dataverse_export', 'conditions', 'may_copy', 'style'}
+    valid_keys = {'type', 'title', 'default', 'note', 'dataverse_export', 'scicat_export', 'conditions', 'may_copy', 'style'}
     schema_keys = set(schema.keys())
     invalid_keys = schema_keys - valid_keys
     if invalid_keys:
@@ -519,18 +629,26 @@ def _validate_bool_schema(schema: dict, path: typing.List[str]) -> None:
         raise ValidationError('default must be bool', path)
     if 'dataverse_export' in schema and not isinstance(schema['dataverse_export'], bool):
         raise ValidationError('dataverse_export must be True or False', path)
-    _validate_note_in_schema(schema, path)
+    if 'scicat_export' in schema and not isinstance(schema['scicat_export'], bool):
+        raise ValidationError('scicat_export must be True or False', path)
+    _validate_note_in_schema(schema, path, all_language_codes=all_language_codes)
 
 
-def _validate_quantity_schema(schema: dict, path: typing.List[str]) -> None:
+def _validate_quantity_schema(
+        schema: typing.Dict[str, typing.Any],
+        path: typing.List[str],
+        *,
+        all_language_codes: typing.Set[str]
+) -> None:
     """
     Validates the given quantity object schema and raises a ValidationError if it is invalid.
 
     :param schema: the sampledb object schema
     :param path: the path to this subschema
+    :param all_language_codes: the set of existing language codes
     :raise ValidationError: if the schema is invalid.
     """
-    valid_keys = {'type', 'title', 'units', 'default', 'note', 'placeholder', 'dataverse_export', 'conditions', 'may_copy', 'style'}
+    valid_keys = {'type', 'title', 'units', 'default', 'note', 'placeholder', 'dataverse_export', 'scicat_export', 'conditions', 'may_copy', 'style', 'display_digits', 'min_magnitude', 'max_magnitude'}
     required_keys = {'type', 'title', 'units'}
     schema_keys = set(schema.keys())
     invalid_keys = schema_keys - valid_keys
@@ -540,20 +658,51 @@ def _validate_quantity_schema(schema: dict, path: typing.List[str]) -> None:
     if missing_keys:
         raise ValidationError('missing keys in schema: {}'.format(missing_keys), path)
 
-    all_language_codes = {
-        language.lang_code
-        for language in get_languages()
-    }
-
-    if not isinstance(schema['units'], str):
-        raise ValidationError('units must be str', path)
-    elif not units_are_valid(schema['units']):
-        raise ValidationError('invalid units', path)
+    if isinstance(schema['units'], str):
+        if not units_are_valid(schema['units']):
+            raise ValidationError('invalid units', path)
+    elif isinstance(schema['units'], list) and len(schema['units']) > 0:
+        dimensionality = None
+        pint_units = []
+        for unit in schema['units']:
+            if not isinstance(unit, str):
+                raise ValidationError('units must be string or list of strings', path)
+            try:
+                quantity = datatypes.Quantity(1.0, units=unit)
+            except pint.UndefinedUnitError:
+                raise ValidationError('invalid units', path)
+            if dimensionality is None:
+                dimensionality = quantity.dimensionality
+            elif quantity.dimensionality != dimensionality:
+                raise ValidationError('units must be for same dimensionality', path)
+            if quantity.pint_units in pint_units:
+                raise ValidationError('units must not be duplicate', path)
+            pint_units.append(quantity.pint_units)
+    else:
+        raise ValidationError('units must be string or list of strings', path)
 
     if 'default' in schema and not isinstance(schema['default'], float) and not isinstance(schema['default'], int):
         raise ValidationError('default must be float or int', path)
+    if 'min_magnitude' in schema and not isinstance(schema['min_magnitude'], float) and not isinstance(schema['min_magnitude'], int):
+        raise ValidationError('min_magnitude must be float or int', path)
+    if 'max_magnitude' in schema and not isinstance(schema['max_magnitude'], float) and not isinstance(schema['max_magnitude'], int):
+        raise ValidationError('max_magnitude must be float or int', path)
+    if 'default' in schema and not math.isfinite(schema['default']):
+        raise ValidationError('default must be a finite number', path)
+    if 'min_magnitude' in schema and not math.isfinite(schema['min_magnitude']):
+        raise ValidationError('min_magnitude must be a finite number', path)
+    if 'max_magnitude' in schema and not math.isfinite(schema['max_magnitude']):
+        raise ValidationError('max_magnitude must be a finite number', path)
+    if 'min_magnitude' in schema and 'max_magnitude' in schema and schema['min_magnitude'] > schema['max_magnitude']:
+        raise ValidationError('max_magnitude must be greater than or equal to min_magnitude', path)
+    if 'min_magnitude' in schema and 'default' in schema and schema['min_magnitude'] > schema['default']:
+        raise ValidationError('default must be greater than or equal to min_magnitude', path)
+    if 'max_magnitude' in schema and 'default' in schema and schema['max_magnitude'] < schema['default']:
+        raise ValidationError('default must be less than or equal to max_magnitude', path)
     if 'dataverse_export' in schema and not isinstance(schema['dataverse_export'], bool):
         raise ValidationError('dataverse_export must be True or False', path)
+    if 'scicat_export' in schema and not isinstance(schema['scicat_export'], bool):
+        raise ValidationError('scicat_export must be True or False', path)
     if 'placeholder' in schema and not isinstance(schema['placeholder'], str) and not isinstance(schema['placeholder'], dict):
         raise ValidationError('placeholder must be str or dict', path)
     if 'placeholder' in schema and isinstance(schema['placeholder'], dict):
@@ -565,18 +714,26 @@ def _validate_quantity_schema(schema: dict, path: typing.List[str]) -> None:
         for placeholder_text in schema['placeholder'].values():
             if not isinstance(placeholder_text, str):
                 raise ValidationError('placeholder must only contain text', path)
-    _validate_note_in_schema(schema, path)
+    if 'display_digits' in schema and (type(schema['display_digits']) is not int or schema['display_digits'] < 0):
+        raise ValidationError('display_digits must be a non-negative int', path)
+    _validate_note_in_schema(schema, path, all_language_codes=all_language_codes)
 
 
-def _validate_sample_schema(schema: dict, path: typing.List[str]) -> None:
+def _validate_sample_schema(
+        schema: typing.Dict[str, typing.Any],
+        path: typing.List[str],
+        *,
+        all_language_codes: typing.Set[str]
+) -> None:
     """
     Validates the given sample object schema and raises a ValidationError if it is invalid.
 
     :param schema: the sampledb object schema
     :param path: the path to this subschema
+    :param all_language_codes: the set of existing language codes
     :raise ValidationError: if the schema is invalid.
     """
-    valid_keys = {'type', 'title', 'note', 'dataverse_export', 'conditions', 'may_copy', 'style'}
+    valid_keys = {'type', 'title', 'note', 'dataverse_export', 'scicat_export', 'conditions', 'may_copy', 'style'}
     required_keys = {'type', 'title'}
     schema_keys = set(schema.keys())
     invalid_keys = schema_keys - valid_keys
@@ -588,18 +745,26 @@ def _validate_sample_schema(schema: dict, path: typing.List[str]) -> None:
 
     if 'dataverse_export' in schema and not isinstance(schema['dataverse_export'], bool):
         raise ValidationError('dataverse_export must be True or False', path)
-    _validate_note_in_schema(schema, path)
+    if 'scicat_export' in schema and not isinstance(schema['scicat_export'], bool):
+        raise ValidationError('scicat_export must be True or False', path)
+    _validate_note_in_schema(schema, path, all_language_codes=all_language_codes)
 
 
-def _validate_measurement_schema(schema: dict, path: typing.List[str]) -> None:
+def _validate_measurement_schema(
+        schema: typing.Dict[str, typing.Any],
+        path: typing.List[str],
+        *,
+        all_language_codes: typing.Set[str]
+) -> None:
     """
     Validates the given measurement object schema and raises a ValidationError if it is invalid.
 
     :param schema: the sampledb object schema
     :param path: the path to this subschema
+    :param all_language_codes: the set of existing language codes
     :raise ValidationError: if the schema is invalid.
     """
-    valid_keys = {'type', 'title', 'note', 'dataverse_export', 'conditions', 'may_copy', 'style'}
+    valid_keys = {'type', 'title', 'note', 'dataverse_export', 'scicat_export', 'conditions', 'may_copy', 'style'}
     required_keys = {'type', 'title'}
     schema_keys = set(schema.keys())
     invalid_keys = schema_keys - valid_keys
@@ -611,18 +776,26 @@ def _validate_measurement_schema(schema: dict, path: typing.List[str]) -> None:
 
     if 'dataverse_export' in schema and not isinstance(schema['dataverse_export'], bool):
         raise ValidationError('dataverse_export must be True or False', path)
-    _validate_note_in_schema(schema, path)
+    if 'scicat_export' in schema and not isinstance(schema['scicat_export'], bool):
+        raise ValidationError('scicat_export must be True or False', path)
+    _validate_note_in_schema(schema, path, all_language_codes=all_language_codes)
 
 
-def _validate_object_reference_schema(schema: dict, path: typing.List[str]) -> None:
+def _validate_object_reference_schema(
+        schema: typing.Dict[str, typing.Any],
+        path: typing.List[str],
+        *,
+        all_language_codes: typing.Set[str]
+) -> None:
     """
     Validates the given object reference object schema and raises a ValidationError if it is invalid.
 
     :param schema: the sampledb object schema
     :param path: the path to this subschema
+    :param all_language_codes: the set of existing language codes
     :raise ValidationError: if the schema is invalid.
     """
-    valid_keys = {'type', 'title', 'note', 'action_type_id', 'action_id', 'dataverse_export', 'conditions', 'may_copy', 'style'}
+    valid_keys = {'type', 'title', 'note', 'action_type_id', 'action_id', 'dataverse_export', 'scicat_export', 'conditions', 'may_copy', 'style'}
     required_keys = {'type', 'title'}
     schema_keys = set(schema.keys())
     invalid_keys = schema_keys - valid_keys
@@ -650,7 +823,9 @@ def _validate_object_reference_schema(schema: dict, path: typing.List[str]) -> N
         raise ValidationError('action_id must be int, None or a list of ints', path)
     if 'dataverse_export' in schema and not isinstance(schema['dataverse_export'], bool):
         raise ValidationError('dataverse_export must be True or False', path)
-    _validate_note_in_schema(schema, path)
+    if 'scicat_export' in schema and not isinstance(schema['scicat_export'], bool):
+        raise ValidationError('scicat_export must be True or False', path)
+    _validate_note_in_schema(schema, path, all_language_codes=all_language_codes)
 
 
 def _validate_notebook_templates(notebook_templates: typing.Any) -> None:
@@ -703,15 +878,21 @@ def _validate_notebook_templates(notebook_templates: typing.Any) -> None:
                     raise ValidationError('notebook template param value must be a list or one of {}'.format(valid_param_values), path)
 
 
-def _validate_user_schema(schema: dict, path: typing.List[str]) -> None:
+def _validate_user_schema(
+        schema: typing.Dict[str, typing.Any],
+        path: typing.List[str],
+        *,
+        all_language_codes: typing.Set[str]
+) -> None:
     """
     Validates the given user object schema and raises a ValidationError if it is invalid.
 
     :param schema: the sampledb object schema
     :param path: the path to this subschema
+    :param all_language_codes: the set of existing language codes
     :raise ValidationError: if the schema is invalid.
     """
-    valid_keys = {'type', 'title', 'note', 'dataverse_export', 'default', 'conditions', 'may_copy', 'style'}
+    valid_keys = {'type', 'title', 'note', 'dataverse_export', 'scicat_export', 'default', 'conditions', 'may_copy', 'style'}
     required_keys = {'type', 'title'}
     schema_keys = set(schema.keys())
     invalid_keys = schema_keys - valid_keys
@@ -722,17 +903,25 @@ def _validate_user_schema(schema: dict, path: typing.List[str]) -> None:
         raise ValidationError('missing keys in schema: {}'.format(missing_keys), path)
     if 'dataverse_export' in schema and not isinstance(schema['dataverse_export'], bool):
         raise ValidationError('dataverse_export must be True or False', path)
-    if 'default' in schema and schema['default'] != 'self':
-        raise ValidationError('default must be "self"', path)
-    _validate_note_in_schema(schema, path)
+    if 'scicat_export' in schema and not isinstance(schema['scicat_export'], bool):
+        raise ValidationError('scicat_export must be True or False', path)
+    if 'default' in schema and (schema['default'] != 'self' and type(schema['default']) is not int):
+        raise ValidationError('default must be "self" or int', path)
+    _validate_note_in_schema(schema, path, all_language_codes=all_language_codes)
 
 
-def _validate_plotly_chart_schema(schema: dict, path: typing.List[str]) -> None:
+def _validate_plotly_chart_schema(
+        schema: typing.Dict[str, typing.Any],
+        path: typing.List[str],
+        *,
+        all_language_codes: typing.Set[str]
+) -> None:
     """
     Validates the given plotly_chart object schema and raises a ValidationError if it is invalid.
 
     :param schema: the sampledb object schema
     :param path: the path to this subschema
+    :param all_language_codes: the set of existing language codes
     :raise ValidationError: if the schema is invalid.
     """
     valid_keys = {'type', 'title', 'note', 'dataverse_export', 'conditions', 'may_copy', 'style'}
@@ -740,4 +929,4 @@ def _validate_plotly_chart_schema(schema: dict, path: typing.List[str]) -> None:
     invalid_keys = schema_keys - valid_keys
     if invalid_keys:
         raise ValidationError('unexpected keys in schema: {}'.format(invalid_keys), path)
-    _validate_note_in_schema(schema, path)
+    _validate_note_in_schema(schema, path, all_language_codes=all_language_codes)
